
/////////////////////////////////////////////////////////////////////////////
//
// Feather 0.1
//
// Alden Dima
// alden.dima@nist.gov
//
// National Institute of Standards and Technology
// Software Diagnostics and Conformance Testing Division
// Gaithersburg, MD USA
// http://www.itl.nist.gov/div897
//
// Feather is released by the National Institute of Standards and Technology
// (NIST), an agency of the U.S. Department of Commerce, Gaithersburg
// MD 20899, USA.  Feather bears no warranty, either express or implied.
// NIST does not assume legal liability nor responsibility for a User's
// use of Feather or the results of such use.
//
// Please note that within the United States, copyright protection, under
// Section 105 of the United States Code, Title 17, is not available for
// any work of the United States Government and/or for any works created
// by United States Government employees.  User acknowledges that Feather
// contains work which was created by NIST employees and is therefore
// in the public domain and not subject to copyright. The User may use,
// distribute, or incorporate Feather provided the User acknowledges this via
// an explicit acknowledgment of NIST-related contributions to the User's   
// work. User also agrees to acknowledge, via an explicit acknowledgment,
// that any modifications or alterations have been made to Feather before
// redistribution.
//
//////////////////////////////////////////////////////////////////////////////

#include <assert.h>
#include <jni.h>
#include <string.h>
#include <stdio.h>
#include <tcl.h>

#define BUF_SIZE 65536

/*
 * Class:     nist_feather_TclInterpreter
 * Method:    destroy
 * Signature: (J)V
 */
JNIEXPORT void JNICALL Java_nist_feather_TclInterpreter_destroy
  (JNIEnv *env, jobject obj, jint interpreter)
{
	Tcl_DeleteInterp((Tcl_Interp *) interpreter);
}

/*
 * Class:     nist_feather_TclInterpreter
 * Method:    eval
 * Signature: (JLjava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_nist_feather_TclInterpreter_eval
  (JNIEnv *env, jobject obj, jint interpreter, jstring script)
{
	jclass xcpt;
	const char *scriptUTF, *interp_result, *resultUTF;
	jboolean isCopy;
	int code;
	jstring result;
	char buf[BUF_SIZE];
	Tcl_Interp *interp = (Tcl_Interp *) interpreter;

	if (!script) {
		xcpt = (*env)->FindClass(env, "java/lang/NullPointerException");
		(*env)->ThrowNew(env, xcpt, "Null script argument");
		return NULL;
	}

	scriptUTF = (*env)->GetStringUTFChars(env, script, &isCopy);

	if (strlen(scriptUTF) >= BUF_SIZE) {
		xcpt = (*env)->FindClass(env, "nist/feather/TclEvalException");
		(void) sprintf(buf, "The script is too long - it must be less than %d bytes.", BUF_SIZE);
		(*env)->ThrowNew(env, xcpt, buf);
		return NULL;
	}

	(void) strncpy(buf, scriptUTF, BUF_SIZE - 1);	
	(*env)->ReleaseStringUTFChars(env, script, scriptUTF);	

	code = Tcl_Eval(interp, buf);
	interp_result = Tcl_GetStringResult(interp);

	/* The Tcl interpreter owns the result - just making sure that it doesn't 
	inadvertently get misused. */
	if (strlen(interp_result) >= BUF_SIZE) {
		xcpt = (*env)->FindClass(env, "nist/feather/TclEvalException");
		(void) sprintf(buf, "The result is too long - it must be less than %d bytes.", BUF_SIZE);
		(*env)->ThrowNew(env, xcpt, buf);
		return NULL;
	}	
	
	resultUTF = strncpy(buf, interp_result, BUF_SIZE);

	/* Just being a little extra paranoid about null pointers for the result string. */
	if (resultUTF) {
		result = (*env)->NewStringUTF(env, resultUTF);
	}
	else {
		result = (*env)->NewStringUTF(env, "");
	}

	if ((*env)->ExceptionOccurred(env)) {
		return NULL;
	}

	if (code != TCL_OK) {
		xcpt = (*env)->FindClass(env, "nist/feather/TclEvalException");
		(*env)->ThrowNew(env, xcpt, (resultUTF) ? resultUTF : "");
	}

	return result;
}

/*
 * Class:     nist_feather_TclInterpreter
 * Method:    evalFile
 * Signature: (JLjava/io/File;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_nist_feather_TclInterpreter_evalFile
  (JNIEnv *env, jobject obj, jint interpreter, jobject file)
{
	jclass fclazz, xcpt;
	jmethodID nameId;
	jstring fname;
	const char *fnameUTF, *interp_result, *resultUTF;
	jboolean isCopy;
	int code;
	jstring result;
	char buf[BUF_SIZE];
	Tcl_Interp *interp = (Tcl_Interp *) interpreter;

	/* Resetting result to avoid a problem with the result "sticking" around when the
	interpreter returns !TCL_OK */
	Tcl_ResetResult(interp);

	if (!file) {
		xcpt = (*env)->FindClass(env, "java/lang/NullPointerException");
		(*env)->ThrowNew(env, xcpt, "Null script file argument");
		return NULL;
	}

	fclazz = (*env)->GetObjectClass(env, file);
	nameId = (*env)->GetMethodID(env, fclazz, "getAbsolutePath", "()Ljava/lang/String;");
	if ((*env)->ExceptionOccurred(env)) {
		return NULL;
	}

	fname = (jstring) (*env)->CallObjectMethod(env, file, nameId);
	if ((*env)->ExceptionOccurred(env)) {
		return NULL;
	}

	fnameUTF = (*env)->GetStringUTFChars(env, fname, &isCopy);
	(void) strncpy( buf, fnameUTF, BUF_SIZE - 1);	
	(*env)->ReleaseStringUTFChars(env, fname, fnameUTF);	

	code = Tcl_EvalFile(interp, buf);
	interp_result = Tcl_GetStringResult(interp);

	/* The Tcl interpreter owns the result - just making sure that it doesn't 
	inadvertently get misused. */	
	if (strlen(interp_result) >= BUF_SIZE) {
		xcpt = (*env)->FindClass(env, "nist/feather/TclEvalException");
		(void) sprintf(buf, "The result is too long - it must be less than %d bytes.", BUF_SIZE);
		(*env)->ThrowNew(env, xcpt, buf);
		return NULL;
	}

	resultUTF = strncpy(buf, interp_result, BUF_SIZE);

	/* Just being a little extra paranoid about null pointers for the result string. */
	if (resultUTF) {
		result = (*env)->NewStringUTF(env, resultUTF);
	}
	else {
		result = (*env)->NewStringUTF(env, "");
	}

	if ((*env)->ExceptionOccurred(env)) {
		return NULL;
	}

	if (code != TCL_OK) {
		resultUTF = resultUTF ? resultUTF : "";
		xcpt = (*env)->FindClass(env, "nist/feather/TclEvalException");
		(*env)->ThrowNew(env, xcpt, resultUTF);
	}

	return result;
}

/*
 * Class:     nist_feather_TclInterpreter
 * Method:    init
 * Signature: ()J
 */
JNIEXPORT jint JNICALL Java_nist_feather_TclInterpreter_init
  (JNIEnv *env, jobject obj)
{
	assert(sizeof(jint) == sizeof(void *));
	return (jint) Tcl_CreateInterp();
}
