
/////////////////////////////////////////////////////////////////////////////
//
// Feather 0.1
//
// Alden Dima
// alden.dima@nist.gov
//
// National Institute of Standards and Technology
// Software Diagnostics and Conformance Testing Division
// Gaithersburg, MD USA
// http://www.itl.nist.gov/div897
//
// Feather is released by the National Institute of Standards and Technology
// (NIST), an agency of the U.S. Department of Commerce, Gaithersburg
// MD 20899, USA.  Feather bears no warranty, either express or implied.
// NIST does not assume legal liability nor responsibility for a User's
// use of Feather or the results of such use.
//
// Please note that within the United States, copyright protection, under
// Section 105 of the United States Code, Title 17, is not available for
// any work of the United States Government and/or for any works created
// by United States Government employees.  User acknowledges that Feather
// contains work which was created by NIST employees and is therefore
// in the public domain and not subject to copyright. The User may use,
// distribute, or incorporate Feather provided the User acknowledges this via
// an explicit acknowledgment of NIST-related contributions to the User's   
// work. User also agrees to acknowledge, via an explicit acknowledgment,
// that any modifications or alterations have been made to Feather before
// redistribution.
//
//////////////////////////////////////////////////////////////////////////////

package nist.feather;

import java.io.File;

/**
 * This class serves as a proxy class for a Tcl interpreter. A new Tcl interpreter is 
 * created when a TclInterpreter object is instantiated. The interpreter maintains its
 * state between calls to its methods. The Tcl interpreter is destroyed when its
 * TclInterpreter object is finalized.
 * 
 * @author Alden Dima
 * @version 0.1
 */
public class TclInterpreter {
	static {
		System.loadLibrary("feather");
	}

    /**
     * Creates a new TclInterpreter proxy object with its corresponding Tcl interpreter.
     */
    public TclInterpreter() {
        interp = init();
    }

	/**
	 * Causes the Tcl interpreter to evaluate a Java string as a script and returns the 
	 * result as a Java string. Can be used to call individual Tcl procs (either
	 * builtin commands or those that have been previously loaded using evalFile).
	 * 
	 * @param script The Java string to be evaluated as a Tcl script. The script length cannot exceed 
	 * 64 KB or a TclEvalException will be thrown.
	 * @return A Java string containing the result of the script evaluation. The result string length
	 * cannot exceed 64 KB or a TclEvalException will be thrown.
	 * @exception nist.feather.TclEvalException
	 */
	public String eval(String script) throws TclEvalException {
	    return eval(interp, script);
	}

	/**
	 * Causes the Tcl interpreter to evaluate a file as a script and returns the 
	 * result as a Java string. Useful as a means to "source" a script so that 
	 * its procs can be called by eval.
	 * 
	 * @param script The Java file object that represents a Tcl script.
	 * @return A Java string containing the result of the script evaluation. The result string length
	 * cannot exceed 64 KB or a TclEvalException will be thrown.
	 * @exception nist.feather.TclEvalException
	 */
	public String evalFile(File script) throws TclEvalException {
	    return evalFile(interp, script);
	}

	/**
	 * Destroys the native Tcl interpreter.
	 */
	protected void finalize() {
	    destroy(interp);
	}

    private int interp;

	private synchronized native int init();

	private synchronized native String eval(int interp, String script)
		throws TclEvalException;

	private synchronized native String evalFile(int interp, File script)
		throws TclEvalException;

    private native void destroy(int interp);
}
